//=============================================================================
// SetFieldSkillId.js (Ver 1.1.0)
//=============================================================================
// [Update History]
// 2019.Sep.05 Ver1.0.0 First Release
// 2019.Sep.16 Ver1.1.0 Add plugin command to get original item information

/*:
 * @plugindesc Set Different Skill ID on Map.
 * @author Sasuke KANNAZUKI
 *
 * @help
 * [Summary]
 * Set different effect battle skill/item on map
 *
 * To set different skill ID, write as follows to skill/item's memo.
 * <FieldSkillId:15>
 * In this case, the skill effect on map becomes the skill whose Id is 15.
 *
 * - When the item isn't a skill but an item, the effect becomes skill.
 * - Be sure to set occasion 'Always', otherwise cannot use it on field.
 * - Recommended usage: changed skill invokes common event.
 * - NOTE: set the same consume MP between original skill and changed skill.
 *
 * [Plugin Commands]
 * SetFieldSkillId setVarIdForOriginal [VarIdForSkill] [VarIdForItem]
 *   Set original item/skill's id to the variable of specified id.
 *   [VarIdForSkill] variable id to store original skill id. when original is
 *     an item, store 0.
 *   [VarIdForItem]  variable id to store item id. when original is a skill,
 *     store 0.
 *   - default variable id is both 0. In this case, doesn't store anywhere.
 *   - these varlables set 0 when not-change item/skill is used.
 *   Example:
 *   SetFieldSkillId setVarIdForOriginal 20 21
 *
 * SetFieldSkillId resetVarIdForOriginal
 *   Cease to store item/skill's id anywhere.
 *
 * [License]
 * this plugin is released under MIT license.
 * http://opensource.org/licenses/mit-license.php
 */

/*:ja
 * @plugindesc 戦闘スキルやアイテムをフィールドで使った時の効果を変更します。
 * @author 神無月サスケ
 *
 * @help
 * ■概要
 * スキルやアイテムのメモに以下のように書いてください：
 * <FieldSkillId:15>
 * この場合、スキル/アイテムの効果は、フィールドではIDが15番のスキルに置き換わります。
 *
 * - アイテムのメモ欄に書いた場合でも、発動するのはスキルです。
 * - 元のスキル/アイテムの「使用可能時」は「常時」にしてください。
 *   さもないと、マップ上では使用できなくなります。
 * - おすすめの使い方：変更後のスキルはコモンイベント起動にする。
 * - 注意：オリジナルのスキルと変更後のスキルは同じ消費MPに設定してください。
 *
 * ■プラグインコマンド
 * SetFieldSkillId setVarIdForOriginal [VarIdForSkill] [VarIdForItem]
 *   マップ上で効果が変わるアイテムやスキルの使用時、元のIDを保存する
 *   変数IDを指定します。
 *   [VarIdForSkill] 元のスキルIDを格納する変数ID。アイテムの時は0を代入。
 *   [VarIdForItem]  元のアイテムIDを格納する変数ID。スキルの時は0を代入。
 *   - デフォルト値はいずれも0です。0が指定された場合、どこにも格納しません。
 *   - 通常のアイテムやスキルを使用した場合、両方に0が格納されます。
 *   例：
 *   SetFieldSkillId setVarIdForOriginal 20 21
 *
 * SetFieldSkillId resetVarIdForOriginal
 *   効果が変わるアイテムやスキル使用時の変数への格納をやめます。
 *
 * ■ライセンス表記
 * このプラグインは MIT ライセンスで配布されます。
 * ご自由にお使いください。
 * http://opensource.org/licenses/mit-license.php
 */

(function() {
  //
  // initialize variables
  //
  var _Game_System_initialize = Game_System.prototype.initialize;
  Game_System.prototype.initialize = function() {
    _Game_System_initialize.call(this);
    this._originalSkillVarId = null;
    this._originalItemVarId = null;
  };

  var _Scene_Battle_start = Scene_Battle.prototype.start;
  Scene_Battle.prototype.start = function() {
    _Scene_Battle_start.call(this);
    this.originalItem = null;
  };

  Game_System.prototype.resetOriginalItem = function() {
    var varId = this._originalItemVarId;
    if (varId) {
      $gameVariables.setValue(varId, 0);
    }
    varId = this._originalSkillVarId;
    if (varId) {
      $gameVariables.setValue(varId, 0);
    }
  };

  //
  // process plugin commands
  //
  var _Game_Interpreter_pluginCommand =
   Game_Interpreter.prototype.pluginCommand;
  Game_Interpreter.prototype.pluginCommand = function(command, args) {
    _Game_Interpreter_pluginCommand.call(this, command, args);
    if (command === 'SetFieldSkillId') {
      switch (args[0]) {
      case 'setVarIdForOriginal':
        var reg = /^(V?)([0-9]+)$/i;
        var s = reg.exec(args[1]);
        if (s) {
          $gameSystem._originalSkillVarId = s[1] ?
           $gameVariables.value(+s[2]) : +s[2];
        } else {
          $gameSystem._originalSkillVarId = 0;
        }
        var i = reg.exec(args[2]);
        if (i) {
          $gameSystem._originalItemVarId = i[1] ? $gameVariables.value(+i[2]) :
           +i[2];
        } else {
          $gameSystem._originalItemVarId = 0;
        }
        break;
      case 'resetVarIdForOriginal':
        $gameSystem.resetOriginalItem();
        $gameSystem._originalSkillVarId = 0;
        $gameSystem._originalItemVarId = 0;
        break;
      }
    }
  };

  //
  // execute item change
  //
  var _Scene_ItemBase_item = Scene_ItemBase.prototype.item;
  Scene_ItemBase.prototype.item = function() {
    var item = _Scene_ItemBase_item.call(this);
    $gameTemp.originalItem = null;
    $gameSystem.resetOriginalItem();
    if (item && item.meta.FieldSkillId) {
      var itemObj = new Game_Item(item);
      if (itemObj.isUsableItem()) {
        if (itemObj.isItem()) {
          $gameTemp.originalItem = item;
          var varId = $gameSystem._originalItemVarId;
          if (varId) {
            $gameVariables.setValue(varId, item.id);
          }
        } else if (itemObj.isSkill()){
          var varId = $gameSystem._originalSkillVarId;
          if (varId) {
            $gameVariables.setValue(varId, item.id);
          }
        }
        var changedSkill = $dataSkills[Number(item.meta.FieldSkillId)];
        if (changedSkill) {
          return changedSkill;
        }
      }
    }
    return item;
  };

  var _Game_Battler_useItem = Game_Battler.prototype.useItem;
  Game_Battler.prototype.useItem = function(item) {
    if ($gameTemp.originalItem) {
      item = $gameTemp.originalItem;
      $gameTemp.originalItem = null;
    }
    _Game_Battler_useItem.call(this, item);
  };

})();
